package errors_test

import (
	"fmt"
	"sync"
	"testing"

	"codeberg.org/gruf/go-errors/v2"
)

var msgTests = []string{
	"hello world",
	"oh no!",
	"aw shit",
}

var formatTests = []struct {
	fmt string
	arg []interface{}
	exp string
}{
	{
		fmt: "wrap: %v",
		arg: []interface{}{errors.New("error")},
		exp: fmt.Sprintf("wrap: %v", errors.New("error")),
	},
	{
		fmt: "%d %d %d",
		arg: []interface{}{0, 1, 2},
		exp: fmt.Sprintf("%d %d %d", 0, 1, 2),
	},
	{
		fmt: "%#v %#v",
		arg: []interface{}{sync.Mutex{}, map[string]string{"hello": "world"}},
		exp: fmt.Sprintf("%#v %#v", sync.Mutex{}, map[string]string{"hello": "world"}),
	},
}

func TestNew(t *testing.T) {
	for _, msg := range msgTests {
		err := errors.New(msg)

		if str := err.Error(); str != msg {
			t.Errorf("unexpected error string: expect=%q actual=%q", msg, str)
		}
	}
}

func TestNewf(t *testing.T) {
	for _, test := range formatTests {
		err := errors.Newf(test.fmt, test.arg...)

		if str := err.Error(); str != test.exp {
			t.Errorf("unexpected error string: expect=%q actual=%q", test.exp, str)
		}
	}
}

func TestWrap(t *testing.T) {
	base := errors.New("base error")

	for _, msg := range msgTests {
		err := errors.Wrap(base, msg)

		if str := err.Error(); str != msg {
			t.Errorf("unexpected error string: expect=%q actual=%q", msg, str)
		}

		if !errors.Comparable(err, base) {
			t.Errorf("wrapped error did not identify as base")
		}
	}
}

func TestWrapf(t *testing.T) {
	base := errors.New("base error")

	for _, test := range formatTests {
		err := errors.Wrapf(base, test.fmt, test.arg...)

		if str := err.Error(); str != test.exp {
			t.Errorf("unexpected error string: expect=%q actual=%q", test.exp, str)
		}

		if !errors.Comparable(err, base) {
			t.Errorf("wrapped error did not identify as base")
		}
	}
}
